@@ -353,6 +353,7 @@ fn resolve_func_index(module: &ModuleInner, name: &str) -> ResolveResult<FuncInd
         module
             .info
             .exports
+            .map
             .get(name)
             .ok_or_else(|| ResolveError::ExportNotFound {
                 name: name.to_string(),
@@ -474,7 +475,7 @@ impl InstanceInner {
 
 impl LikeNamespace for Instance {
     fn get_export(&self, name: &str) -> Option<Export> {
-        let export_index = self.module.info.exports.get(name)?;
+        let export_index = self.module.info.exports.map.get(name)?;
 
         Some(self.inner.get_export_from_index(&self.module, export_index))
     }
@@ -491,7 +492,7 @@ impl LikeNamespace for Instance {
 use std::rc::Rc;
 impl LikeNamespace for Rc<Instance> {
     fn get_export(&self, name: &str) -> Option<Export> {
-        let export_index = self.module.info.exports.get(name)?;
+        let export_index = self.module.info.exports.map.get(name)?;
 
         Some(self.inner.get_export_from_index(&self.module, export_index))
     }
@@ -508,7 +509,7 @@ impl LikeNamespace for Rc<Instance> {
 impl LikeNamespace for Arc<Mutex<Instance>> {
     fn get_export(&self, name: &str) -> Option<Export> {
         let instance = self.lock().unwrap();
-        let export_index = instance.module.info.exports.get(name)?;
+        let export_index = instance.module.info.exports.map.get(name)?;
 
         Some(
             instance
@@ -780,6 +781,7 @@ impl<'a> Exportable<'a> for Memory {
             module
                 .info
                 .exports
+                .map
                 .get(name)
                 .ok_or_else(|| ResolveError::ExportNotFound {
                     name: name.to_string(),
@@ -801,6 +803,7 @@ impl<'a> Exportable<'a> for Table {
             module
                 .info
                 .exports
+                .map
                 .get(name)
                 .ok_or_else(|| ResolveError::ExportNotFound {
                     name: name.to_string(),
@@ -822,6 +825,7 @@ impl<'a> Exportable<'a> for Global {
             module
                 .info
                 .exports
+                .map
                 .get(name)
                 .ok_or_else(|| ResolveError::ExportNotFound {
                     name: name.to_string(),
