@@ -3,9 +3,10 @@
 
 use crate::{memory::MemoryType, module::ModuleInfo, structures::TypedIndex, units::Pages};
 use std::{borrow::Cow, convert::TryFrom};
+use borsh::{BorshDeserialize, BorshSerialize};
 
 /// Represents a WebAssembly type.
-#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq, Hash)]
+#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq, Hash, BorshSerialize, BorshDeserialize)]
 pub enum Type {
     /// The `i32` type.
     I32,
@@ -29,7 +30,7 @@ impl std::fmt::Display for Type {
 ///
 /// As the number of types in WebAssembly expand,
 /// this structure will expand as well.
-#[derive(Serialize, Deserialize, Debug, Clone, PartialEq)]
+#[derive(Serialize, Deserialize, Debug, Clone, PartialEq, BorshSerialize, BorshDeserialize)]
 pub enum Value {
     /// The `i32` type.
     I32(i32),
@@ -246,7 +247,7 @@ macro_rules! convert_value_impl {
 convert_value_impl!(u8, i8, u16, i16, u32, i32, u64, i64, f32, f64);
 
 /// Kinds of element types.
-#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq, BorshSerialize, BorshDeserialize)]
 pub enum ElementType {
     /// Any wasm function.
     Anyfunc,
@@ -254,7 +255,7 @@ pub enum ElementType {
 
 /// Describes the properties of a table including the element types, minimum and optional maximum,
 /// number of elements in the table.
-#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq, BorshSerialize, BorshDeserialize)]
 pub struct TableDescriptor {
     /// Type of data stored in this table.
     pub element: ElementType,
@@ -278,7 +279,7 @@ impl TableDescriptor {
 /// A const value initializer.
 /// Over time, this will be able to represent more and more
 /// complex expressions.
-#[derive(Serialize, Deserialize, Debug, Clone, PartialEq)]
+#[derive(Serialize, Deserialize, Debug, Clone, PartialEq, BorshSerialize, BorshDeserialize)]
 pub enum Initializer {
     /// Corresponds to a `const.*` instruction.
     Const(Value),
@@ -287,7 +288,7 @@ pub enum Initializer {
 }
 
 /// Describes the mutability and type of a Global
-#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq, BorshSerialize, BorshDeserialize)]
 pub struct GlobalDescriptor {
     /// Mutable flag.
     pub mutable: bool,
@@ -296,7 +297,7 @@ pub struct GlobalDescriptor {
 }
 
 /// A wasm global.
-#[derive(Serialize, Deserialize, Debug, Clone)]
+#[derive(Serialize, Deserialize, Debug, Clone, BorshSerialize, BorshDeserialize)]
 pub struct GlobalInit {
     /// Global descriptor.
     pub desc: GlobalDescriptor,
@@ -305,7 +306,7 @@ pub struct GlobalInit {
 }
 
 /// A wasm memory descriptor.
-#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Serialize, Deserialize, Debug, Clone, Copy, PartialEq, Eq, BorshSerialize, BorshDeserialize)]
 pub struct MemoryDescriptor {
     /// The minimum number of allowed pages.
     pub minimum: Pages,
@@ -353,7 +354,7 @@ impl MemoryDescriptor {
 
 /// The signature of a function that is either implemented
 /// in a wasm module or exposed to wasm by the host.
-#[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq, Hash)]
+#[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq, Hash, BorshSerialize, BorshDeserialize)]
 pub struct FuncSig {
     params: Cow<'static, [Type]>,
     returns: Cow<'static, [Type]>,
@@ -426,7 +427,7 @@ pub trait LocalImport {
 macro_rules! define_map_index {
     ($ty:ident) => {
         /// Typed Index
-        #[derive(Serialize, Deserialize)]
+        #[derive(Serialize, Deserialize, BorshSerialize, BorshDeserialize)]
         #[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
         pub struct $ty (u32);
         impl TypedIndex for $ty {
@@ -506,7 +507,7 @@ define_local_or_import![
 ];
 
 /// Index for signature.
-#[derive(Serialize, Deserialize, Debug, Copy, Clone, PartialEq, Eq, Hash)]
+#[derive(Serialize, Deserialize, Debug, Copy, Clone, PartialEq, Eq, Hash, BorshSerialize, BorshDeserialize)]
 pub struct SigIndex(u32);
 impl TypedIndex for SigIndex {
     #[doc(hidden)]
